@@ -2,9 +2,9 @@
 This module contains classes which define adjustments to model parameters.
 """
 from abc import ABC, abstractmethod
-from typing import Callable, Union, Any
+from typing import Callable, Union
 
-import numpy as np
+from summer.parameters import get_model_param_value
 
 FlowParam = Union[float, Callable[[float], float]]
 
@@ -18,7 +18,9 @@ class BaseAdjustment(ABC):
         self.param = param
 
     @abstractmethod
-    def get_new_value(self, value: float, computed_values: dict, time: float) -> float:
+    def get_new_value(
+        self, value: float, computed_values: dict, time: float, parameters: dict = None
+    ) -> float:
         pass
 
     def _is_equal(self, adj):
@@ -29,7 +31,7 @@ class BaseAdjustment(ABC):
         return f"<{self.__class__.__name__} '{self.param}'>"
 
     def __hash__(self):
-        return hash((type(self),self.param))
+        return hash((type(self), self.param))
 
     def __eq__(self, o: object) -> bool:
         return self._is_equal(o)
@@ -56,7 +58,9 @@ class Multiply(BaseAdjustment):
 
     """
 
-    def get_new_value(self, value: float, computed_values: dict, time: float) -> float:
+    def get_new_value(
+        self, value: float, computed_values: dict, time: float, parameters: dict = None
+    ) -> float:
         """
         Returns the adjusted value for a given time.
 
@@ -68,7 +72,7 @@ class Multiply(BaseAdjustment):
             float: The new, adjusted value.
 
         """
-        return self.param(time, computed_values) * value if callable(self.param) else self.param * value
+        return get_model_param_value(self.param, time, computed_values, parameters) * value
 
 
 class Overwrite(BaseAdjustment):
@@ -84,14 +88,17 @@ class Overwrite(BaseAdjustment):
 
             adjust.Overwrite(1.5)
 
-        Create an adjustment to overwrite the previous value with the value of a time varying function::
+        Create an adjustment to overwrite the previous value with the value of a
+        time varying function::
 
             arbitrary_function = lambda time: 2 * time + 1
             adjust.Overwrite(arbitrary_function)
 
     """
 
-    def get_new_value(self, value: float, computed_values: dict, time: float) -> float:
+    def get_new_value(
+        self, value: float, computed_values: dict, time: float, parameters: dict = None
+    ) -> float:
         """
         Returns the adjusted value for a given time.
 
@@ -103,44 +110,15 @@ class Overwrite(BaseAdjustment):
             float: The new, adjusted value.
 
         """
-        return self.param(time, computed_values) if callable(self.param) else self.param
+        return get_model_param_value(self.param, time, computed_values, parameters)
 
-class AdjustmentComponent:
-    def __init__(self, system: str, data: Any):
-        """Adjustment is a component of an AdjustmentSystem
-        The component does not compute a value directly, rather it contains
-        the data that the system can use to compute all its values
-        in a vectorized fashion
 
-        Args:
-            system (str): Name matching a system registered via add_adjustment_system
-            data (Any): Data of any type matching the system's interface 
-        """
-        self.system = system
-        self.data = data
+def enforce_wrapped(value, allowed, wrap):
+    if any([isinstance(value, t) for t in allowed]):
+        return value
+    else:
+        return wrap(value)
 
-class AdjustmentSystem:
-    def __init__(self):
-        pass
 
-    @abstractmethod
-    def prepare_to_run(self, component_data: list):
-        """Do any preparation required before a run here.
-        component_data is an ordered list of all components belonging to this system
-        The system is expected to return an ndarray of matching size, in the same
-        order, with one flow weight adjustment per component
-
-        Args:
-            component_data (list): Initialization data for the system
-        """
-        pass
-
-    @abstractmethod
-    def get_weights_at_time(self, time: float, computed_values: dict) -> np.ndarray:
-        """[summary]
-
-        Args:
-            time ([type]): [description]
-            computed_values ([type]): [description]
-        """
-        pass
\ No newline at end of file
+def enforce_multiply(value):
+    return enforce_wrapped(value, [Multiply, Overwrite, type(None)], Multiply)
