@@ -6,21 +6,27 @@ import logging
 from datetime import datetime
 from collections import OrderedDict
 from typing import Callable, Dict, List, Optional, Tuple
+from warnings import warn
 
 import networkx
 import numpy as np
 import pandas as pd
 
+from computegraph.utils import is_var
+
 import summer.flows as flows
 from summer import stochastic
-from summer.adjust import FlowParam, AdjustmentSystem
+from summer.adjust import BaseAdjustment, FlowParam, Multiply
 from summer.compartment import Compartment
 from summer.compute import ComputedValueProcessor
 from summer.derived_outputs import DerivedOutputRequest, calculate_derived_outputs
+from summer.parameters import params
 from summer.runner import ReferenceRunner, VectorizedRunner
 from summer.solver import SolverType, solve_ode
 from summer.stratification import Stratification
 from summer.utils import get_scenario_start_index, ref_times_to_dti
+from summer.population import get_unique_strat_groups, filter_by_strata
+from summer.tracker import ModelBuildTracker, ActionType
 
 logger = logging.getLogger()
 
@@ -38,28 +44,34 @@ class CompartmentalModel:
 
     This model defines a set of compartments which each contain a population.
     Disease dynamics are defined by a set of flows which link the compartments together.
-    The model is run over a period of time, starting from some initial conditions to predict the future state of a disease.
+    The model is run over a period of time, starting from some initial conditions to
+    predict the future state of a disease.
 
     Args:
         times: The start and end times. ***
         compartments: The compartments to simulate.
         infectious_compartments: The compartments which are counted as infectious.
-        time_step (optional): The timesteps to return results for. This request does not affect the ODE solver, but is used for the stochastic solver. Defaults to ``1``.
+        time_step (optional): The timesteps to return results for. This request does not
+        affect the ODE solver, but is used for the stochastic solver. Defaults to ``1``.
 
     Attributes:
         times (np.ndarray): The times that the model will simulate.
         compartments (List[Compartment]): The model's compartments.
         initial_population (np.ndarray): The model's starting population. The indices of this
-            array match to the ``compartments`` attribute. This is zero by default, but should be set with the ``set_initial_population`` method.
-        outputs (np.ndarray): The values of each compartment for each requested timestep. For ``C`` compartments and
-            ``T`` timesteps this will be a ``TxC`` matrix. The column indices of this array will match up with ``compartments`` and the row indices will match up with ``times``.
-        derived_outputs (Dict[str, np.ndarray]): Additional results that are calculated from ``outputs`` for each timestep.
+            array match to the ``compartments`` attribute. This is zero by default, but should
+            be set with the ``set_initial_population`` method.
+        outputs (np.ndarray): The values of each compartment for each requested timestep.
+            For ``C`` compartments and ``T`` timesteps this will be a ``TxC`` matrix.
+            The column indices of this array will match up with ``compartments`` and the
+            row indices will match up with ``times``.
+        derived_outputs (Dict[str, np.ndarray]): Additional results that are calculated from
+            ``outputs`` for each timestep.
 
 
     """
 
     _DEFAULT_DISEASE_STRAIN = "default"
-    _DEFAULT_MIXING_MATRIX = np.array([[1.]])
+    _DEFAULT_MIXING_MATRIX = np.array([[1.0]])
     _DEFAULT_BACKEND = BackendType.VECTORIZED
 
     def __init__(
@@ -68,7 +80,8 @@ class CompartmentalModel:
         compartments: List[str],
         infectious_compartments: List[str],
         timestep: float = 1.0,
-        ref_date: datetime = None
+        ref_date: datetime = None,
+        takes_params: bool = False,
     ):
         start_t, end_t = times
         assert end_t > start_t, "End time must be greater than start time"
@@ -115,25 +128,33 @@ class CompartmentalModel:
         # Whitelist of 'derived outputs' to evaluate
         self._derived_outputs_whitelist = []
 
-        # Adjustment systems
-        self._adjustment_systems = {}
-
         # Map of (runtime) computed values
         self._computed_value_processors = OrderedDict()
+        self._computed_values_graph_dict = {}
 
         # Init baseline model to None; can be set via set_baseline if running as a scenario
         self._baseline = None
 
-        # Mixing matrices: a list of square arrays, or functions, used to calculate force of infection.
+        # Mixing matrices: a list of square arrays, or functions, used to calculate
+        # force of infection.
         self._mixing_matrices = []
-        # Mixing categories: a list of dicts that knows the strata required to match a row in the mixing matrix.
+        # Mixing categories: a list of dicts that knows the strata required to match a row in
+        # the mixing matrix.
         self._mixing_categories = [{}]
-        # Strains: a list of the different sub-categories ('strains') of the disease that we are modelling.
+        # Strains: a list of the different sub-categories ('strains') of the disease that we
+        # are modelling.
         self._disease_strains = [self._DEFAULT_DISEASE_STRAIN]
 
         self._update_compartment_indices()
 
-        self._enable_cache = True
+        # Track the actions we take building this model
+        self.tracker = ModelBuildTracker()
+
+        if takes_params:
+            self._defer_actions = True
+        else:
+            self._defer_actions = False
+        self._finalized = False
 
     def _update_compartment_indices(self):
         """
@@ -148,6 +169,10 @@ class CompartmentalModel:
         for flow in self._flows:
             flow.update_compartment_indices(compartment_idx_lookup)
 
+    def _assert_not_finalized(self):
+        error_msg = "Cannot make changes to model that is already finalized"
+        assert not self._finalized, error_msg
+
     def set_initial_population(self, distribution: Dict[str, float]):
         """
         Sets the initial population of the model, which is zero by default.
@@ -156,17 +181,27 @@ class CompartmentalModel:
             distribution: A map of populations to be assigned to compartments.
 
         """
+        self._assert_not_finalized()
+
         error_msg = "Cannot set initial population after the model has been stratified"
         assert not self._stratifications, error_msg
-        for idx, comp in enumerate(self.compartments):
-            pop = distribution.get(comp.name, 0)
-            assert pop >= 0, f"Population for {comp.name} cannot be negative: {pop}"
-            self.initial_population[idx] = pop
+        self._initial_population_distribution = distribution
+
+        if isinstance(distribution, dict):
+            for idx, comp in enumerate(self.compartments):
+                pop = distribution.get(comp.name, 0)
+                assert pop >= 0, f"Population for {comp.name} cannot be negative: {pop}"
+                self.initial_population[idx] = pop
+
+            self._original_init_population = self.initial_population.copy()
+
+        elif not is_var(distribution, "parameters"):
+            raise TypeError("Initial population must be dict or Parameter", distribution)
 
     def set_validation_enabled(self, validate: bool):
         """
         Set this to False in order to turn of some (potentially expensive) runtime validation
-        E.g. In calibration, leave it enabled for the first iteration, to catch any structural 
+        E.g. In calibration, leave it enabled for the first iteration, to catch any structural
         model issues, but then disable for subsequent iterations
         """
         self._should_validate = validate
@@ -174,7 +209,6 @@ class CompartmentalModel:
     def _set_derived_outputs_index_cache(self, idx_cache: dict):
         self._derived_outputs_idx_cache = idx_cache
 
-
     """
     Adding flows
     """
@@ -189,14 +223,16 @@ class CompartmentalModel:
     ):
         """
         Adds a crude birth rate flow to the model.
-        The number of births will be determined by the product of the birth rate and total population.
+        The number of births will be determined by the product of the birth rate
+        and total population.
 
         Args:
             name: The name of the new flow.
             birth_rate: The fractional crude birth rate per timestep.
             dest: The name of the destination compartment.
             dest_strata (optional): A whitelist of strata to filter the destination compartments.
-            expected_flow_count (optional): Used to assert that a particular number of flows are created.
+            expected_flow_count (optional): Used to assert that a particular number of flows
+                                            are created.
 
         """
         is_already_birth_flow = any(
@@ -233,7 +269,8 @@ class CompartmentalModel:
             name: The name of the new flow.
             dest: The name of the destination compartment.
             dest_strata (optional): A whitelist of strata to filter the destination compartments.
-            expected_flow_count (optional): Used to assert that a particular number of flows are created.
+            expected_flow_count (optional): Used to assert that a particular number of flows
+                                            are created.
 
         """
         # Only allow a single replacement flow to be added to the model.
@@ -261,23 +298,45 @@ class CompartmentalModel:
         name: str,
         num_imported: FlowParam,
         dest: str,
+        split_imports: bool,
         dest_strata: Optional[Dict[str, str]] = None,
         expected_flow_count: Optional[int] = None,
     ):
         """
-        Adds an importation flow to the model, where people enter the destination compartment from outside the system.
+        Adds an importation flow to the model, where people enter the destination compartment from
+        outside the system.
         The number of people imported per timestep is completely determined by ``num_imported``.
 
         Args:
             name: The name of the new flow.
             num_imported: The number of people arriving per timestep.
             dest: The name of the destination compartment.
+            split_imports: Whether to split num_imported amongst existing destination
+                           compartments (True), or add the full value to each (False)
             dest_strata (optional): A whitelist of strata to filter the destination compartments.
-            expected_flow_count (optional): Used to assert that a particular number of flows are created.
+            expected_flow_count (optional): Used to assert that a particular number of flows are
+                                            created.
 
         """
+
+        dest_strata = dest_strata or {}
+        dest_comps = [c for c in self.compartments if c.is_match(dest, dest_strata)]
+
+        if split_imports:
+            # Adjust each of the flows so they are split equally between dest_comps
+            adjustments = [Multiply(1.0 / len(dest_comps))]
+        else:
+            # No adjustment - flow to each dest will be num_imported
+            adjustments = None
+
         self._add_entry_flow(
-            flows.ImportFlow, name, num_imported, dest, dest_strata, expected_flow_count
+            flows.ImportFlow,
+            name,
+            num_imported,
+            dest,
+            dest_strata,
+            expected_flow_count,
+            adjustments,
         )
 
     def _add_entry_flow(
@@ -288,12 +347,14 @@ class CompartmentalModel:
         dest: str,
         dest_strata: Optional[Dict[str, str]],
         expected_flow_count: Optional[int],
+        adjustments: List[BaseAdjustment] = None,
     ):
+        self._assert_not_finalized()
         dest_strata = dest_strata or {}
         dest_comps = [c for c in self.compartments if c.is_match(dest, dest_strata)]
         new_flows = []
         for dest_comp in dest_comps:
-            flow = flow_cls(name, dest_comp, param)
+            flow = flow_cls(name, dest_comp, param, adjustments=adjustments)
             new_flows.append(flow)
 
         self._validate_expected_flow_count(expected_flow_count, new_flows)
@@ -315,7 +376,8 @@ class CompartmentalModel:
             death_rate: The fractional death rate per timestep.
             source: The name of the source compartment.
             source_strata (optional): A whitelist of strata to filter the source compartments.
-            expected_flow_count (optional): Used to assert that a particular number of flows are created.
+            expected_flow_count (optional): Used to assert that a particular number of flows are
+                                            created.
 
         """
         self._add_exit_flow(
@@ -345,10 +407,17 @@ class CompartmentalModel:
             List[str]: The names of the flows added.
 
         """
+        warn(
+            "Feature is scheduled for deprecation",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+
         # Only allow a single universal death flow with a given name to be added to the model.
         is_already_used = any([f.name.startswith(base_name) for f in self._flows])
         if is_already_used:
-            msg = f"There is already a universal death flow called '{base_name}' in this model, cannot add a second."
+            msg = f"There is already a universal death flow called '{base_name}' in this model, \
+                cannot add a second."
             raise ValueError(msg)
 
         flow_names = []
@@ -375,6 +444,7 @@ class CompartmentalModel:
         source_strata: Optional[Dict[str, str]],
         expected_flow_count: Optional[int],
     ):
+        self._assert_not_finalized()
         source_strata = source_strata or {}
         source_comps = [c for c in self.compartments if c.is_match(source, source_strata)]
         new_flows = []
@@ -397,7 +467,8 @@ class CompartmentalModel:
     ):
         """
         Adds a flow that infects people using an "infection frequency" contact rate, which is
-        when the infectious multiplier is determined by the proportion of infectious people to the total population.
+        when the infectious multiplier is determined by the proportion of infectious people to the
+        total population.
 
         Args:
             name: The name of the new flow.
@@ -406,7 +477,8 @@ class CompartmentalModel:
             dest: The name of the destination compartment.
             source_strata (optional): A whitelist of strata to filter the source compartments.
             dest_strata (optional): A whitelist of strata to filter the destination compartments.
-            expected_flow_count (optional): Used to assert that a particular number of flows are created.
+            expected_flow_count (optional): Used to assert that a particular number of flows are
+                                            created.
 
         """
         self._add_transition_flow(
@@ -442,7 +514,8 @@ class CompartmentalModel:
             dest: The name of the destination compartment.
             source_strata (optional): A whitelist of strata to filter the source compartments.
             dest_strata (optional): A whitelist of strata to filter the destination compartments.
-            expected_flow_count (optional): Used to assert that a particular number of flows are created.
+            expected_flow_count (optional): Used to assert that a particular number of flows are
+                                            created.
 
         """
         self._add_transition_flow(
@@ -468,8 +541,8 @@ class CompartmentalModel:
         expected_flow_count: Optional[int] = None,
     ):
         """
-        Adds a flow transfers people from a source to a destination based on the population of the source
-        compartment and the fractional flow rate.
+        Adds a flow transferring people from a source to a destination based on the population of
+        the source compartment and the fractional flow rate.
 
         Args:
             name: The name of the new flow.
@@ -478,7 +551,8 @@ class CompartmentalModel:
             dest: The name of the destination compartment.
             source_strata (optional): A whitelist of strata to filter the source compartments.
             dest_strata (optional): A whitelist of strata to filter the destination compartments.
-            expected_flow_count (optional): Used to assert that a particular number of flows are created.
+            expected_flow_count (optional): Used to assert that a particular number of flows are
+                                            created.
 
         """
         self._add_transition_flow(
@@ -503,8 +577,9 @@ class CompartmentalModel:
         expected_flow_count: Optional[int] = None,
     ):
         """
-        A flow that transfers people from a source to a destination based on a user-defined function.
-        This can be used to define more complex flows if required. See `flows.FunctionFlow` for more details on the arguments to the function.
+        A flow that transfers people from a source to a destination based on a user-defined function
+        This can be used to define more complex flows if required. See `flows.FunctionFlow` for more
+        details on the arguments to the function.
 
         Args:
             name: The name of the new flow.
@@ -513,7 +588,8 @@ class CompartmentalModel:
             dest: The name of the destination compartment.
             source_strata (optional): A whitelist of strata to filter the source compartments.
             dest_strata (optional): A whitelist of strata to filter the destination compartments.
-            expected_flow_count (optional): Used to assert that a particular number of flows are created.
+            expected_flow_count (optional): Used to assert that a particular number of flows are
+                                            created.
 
         """
         self._add_transition_flow(
@@ -539,6 +615,7 @@ class CompartmentalModel:
         expected_flow_count: Optional[int],
         find_infectious_multiplier=None,
     ):
+        self._assert_not_finalized()
         source_strata = source_strata or {}
         dest_strata = dest_strata or {}
 
@@ -546,8 +623,9 @@ class CompartmentalModel:
         source_comps = self.get_matching_compartments(source, source_strata)
 
         num_dest = len(dest_comps)
-        num_source = len(dest_comps)
-        msg = f"Expected equal number of source and dest compartments, but got {num_source} source and {num_dest} dest."
+        num_source = len(source_comps)
+        msg = f"Expected equal number of source and dest compartments, but got {num_source} source \
+                and {num_dest} dest."
         assert num_dest == num_source, msg
         new_flows = []
         for source_comp, dest_comp in zip(source_comps, dest_comps):
@@ -592,7 +670,7 @@ class CompartmentalModel:
             name_map[n] = [c for c in self.compartments if c.name == n]
         self._compartment_name_map = name_map
 
-    def get_matching_compartments(self, name, strata):
+    def get_matching_compartments(self, name: str, strata: dict):
         name_query = self._compartment_name_map[name]
 
         if not len(strata):
@@ -601,6 +679,17 @@ class CompartmentalModel:
             _strata = frozenset(strata.items())
             return [c for c in name_query if c._has_strata(_strata)]
 
+    def _get_matching_compartments(self, name: str, strata: frozenset):
+        """
+        Optimized version of above method that uses frozenset directly
+        """
+        name_query = self._compartment_name_map[name]
+
+        if not len(strata):
+            return name_query
+        else:
+            return [c for c in name_query if c._has_strata(strata)]
+
     """
     Stratifying the model
     """
@@ -613,6 +702,7 @@ class CompartmentalModel:
             strat: The stratification to apply.
 
         """
+        self._assert_not_finalized()
         # Enable/disable runtime assertions for strat
         strat._validate = self._should_validate
 
@@ -630,7 +720,8 @@ class CompartmentalModel:
                 self._strata_exist(dest_strata)
 
         # Validate infectiousness adjustments.
-        msg = "All stratification infectiousness adjustments must refer to a compartment that is present in model."
+        msg = "All stratification infectiousness adjustments must refer to a compartment that is \
+               present in model."
         assert all(
             [c in self._original_compartment_names for c in strat.infectiousness_adjustments.keys()]
         ), msg
@@ -651,16 +742,22 @@ class CompartmentalModel:
 
         if strat.is_strain():
             # Track disease strain names, overriding default values.
-            msg = "An infection strain stratification has already been applied, cannot use this more than once."
+            msg = "An infection strain stratification has already been applied, cannot use this \
+                   more than once."
             assert not any([s.is_strain() for s in self._stratifications]), msg
             self._disease_strains = strat.strata
 
         # Stratify compartments, split according to split_proportions
         prev_compartment_names = copy.copy(self.compartments)
         self.compartments = strat._stratify_compartments(self.compartments)
-        self.initial_population = strat._stratify_compartment_values(
-            prev_compartment_names, self.initial_population
-        )
+        if not self._defer_actions or self._finalized:
+            try:
+                self.initial_population = strat._stratify_compartment_values(
+                    prev_compartment_names, self.initial_population
+                )
+            except Exception as e:
+                logger.critical("Parameterized models must set takes_params in constructor")
+                raise e
 
         # Update the cache of compartment names; these need to correct whenever we add a new flow
         self._update_compartment_name_map()
@@ -683,7 +780,8 @@ class CompartmentalModel:
             # It's assumed that both ages and model timesteps are in years.
             ages = list(sorted(map(int, strat.strata)))
 
-            # Only allow age stratification to be applied with complete stratifications, because everyone has an age.
+            # Only allow age stratification to be applied with complete stratifications,
+            # because everyone has an age.
             msg = "Mixing matrices only allowed for full stratification."
             assert strat.compartments == self._original_compartment_names, msg
 
@@ -707,6 +805,8 @@ class CompartmentalModel:
 
         self._stratifications.append(strat)
 
+        self.tracker.append_action(ActionType.STRATIFY, strat=strat)
+
     def _strata_exist(self, strata: dict):
         """
         Verify whether all the strata exist within the model
@@ -721,6 +821,51 @@ class CompartmentalModel:
                     if v not in s.strata:
                         raise ValueError(f"Invalid stratum {v} for {s}")
 
+    def adjust_population_split(self, strat: str, dest_filter: dict, proportions: dict):
+        """Adjust the initial population to redistribute the population for a particular
+        stratification, over a subset of some other strata
+
+        Args:
+            strat (str): The stratification to redistribute over
+            dest_filter (dict): Subset of (other) strata to filter the split by
+            proportions (dict): Proportions of new split (must have all strata specified)
+
+        """
+        self._assert_not_finalized()
+
+        msg = f"No stratification {strat} found in model"
+        assert strat in [s.name for s in self._stratifications], msg
+
+        model_strat = [s for s in self._stratifications if s.name == strat][0]
+
+        msg = "All strata must be specified in proportions"
+        assert set(model_strat.strata) == set(proportions), msg
+
+        msg = "Proportions must sum to 1.0"
+        np.testing.assert_allclose(sum(proportions.values()), 1.0, err_msg=msg)
+
+        strat_comps = [c for c in self.compartments if strat in c.strata]
+        # Filter by only the subset we're setting in split_map
+        strat_comps = filter_by_strata(strat_comps, dest_filter)
+
+        usg = get_unique_strat_groups(strat_comps, strat)
+
+        for g in usg:
+            mcomps = self._get_matching_compartments(g.name, g.strata)
+            idx = [c.idx for c in mcomps]
+            total = self.initial_population[idx].sum()
+            for c in mcomps:
+                k = c.strata[strat]
+                target_prop = proportions[k]
+                self.initial_population[c.idx] = total * target_prop
+
+        self.tracker.append_action(
+            ActionType.ADJUST_POP_SPLIT,
+            strat=strat,
+            dest_filter=dest_filter,
+            proportions=proportions,
+        )
+
     """
     Running the model
     """
@@ -730,27 +875,36 @@ class CompartmentalModel:
         solver: str = SolverType.SOLVE_IVP,
         backend: str = _DEFAULT_BACKEND,
         backend_args: dict = None,
+        parameters: dict = None,
         **kwargs,
     ):
         """
         Runs the model over the provided time span, calculating the outputs and the derived outputs.
-        The model calculates the outputs by solving an ODE which is defined by the initial population and the inter-compartmental flows.
+        The model calculates the outputs by solving an ODE which is defined by the initial
+        population and the inter-compartmental flows.
 
-        **Note**: The default ODE solver used to produce the model's outputs does not necessarily evaluate every requested timestep. This adaptive
-        solver can skip over times, or double back when trying to characterize the ODE. The final results are produced by interpolating the
-        solution produced by the ODE solver. This means that model dynamics that only occur in short time periods may not be reflected in the outputs.
+        **Note**: The default ODE solver used to produce the model's outputs does not necessarily
+        evaluate every requested timestep. This adaptive
+        solver can skip over times, or double back when trying to characterize the ODE. The final
+        results are produced by interpolating the
+        solution produced by the ODE solver. This means that model dynamics that only occur in
+        short time periods may not be reflected in the outputs.
 
         Args:
             solver (optional): The ODE solver to use, defaults to SciPy's IVP solver.
-            **kwargs (optional): Extra arguments to supplied to the solver, see ``summer.solver`` for details.
+            **kwargs (optional): Extra arguments to supplied to the solver, see ``summer.solver``
+            for details.
 
         """
 
-        # Ensure we call this before model runs, since it is now disabled inside individual flow constructors
+        # Ensure we call this before model runs, since it is now disabled inside individual
+        # flow constructors
         self._update_compartment_indices()
 
+        self._finalized = True
+
         self._set_backend(backend, backend_args)
-        self._backend.prepare_to_run()
+        self._backend.prepare_to_run(parameters)
 
         if solver == SolverType.STOCHASTIC:
             # Run the model in 'stochastic mode'.
@@ -796,7 +950,8 @@ class CompartmentalModel:
         Uses an ODE interpretation of flow rates.
         """
 
-        # Calculate the outputs (compartment sizes) by solving the ODE defined by _get_compartment_rates().
+        # Calculate the outputs (compartment sizes) by solving the ODE defined by
+        # _get_compartment_rates().
         self.outputs = solve_ode(
             solver,
             self._backend.get_compartment_rates,
@@ -812,8 +967,8 @@ class CompartmentalModel:
         A random seed (eg. 12345, 1337) can be provided to ensure the same results are produced.
 
         With this approach we represent a discrete number of people, unlike in the ODE solver, there
-        is way to have of 'half a person' or a compartment size of 101.23. Both compartment sizes and
-        flows are discrete.
+        is way to have of 'half a person' or a compartment size of 101.23. Both compartment sizes
+        and flows are discrete.
 
         We use an stochastic interpretation of flow rates.
         The flow rates from _get_rates() are used to calculate the probability of an person
@@ -822,7 +977,8 @@ class CompartmentalModel:
             - moving between compartments (transitions, infections)
 
         There are two sampling methods used:
-            - Transition and exit flows are sampled from an exponential distribution using a multinomial
+            - Transition and exit flows are sampled from an exponential distribution using a
+              multinomial
             - Entry flows are sampled froma  Poisson distribution
 
         The main difference is that there is no 'source' compartment for entry flows.
@@ -860,7 +1016,8 @@ class CompartmentalModel:
             # because we handle them separately with different methods.
 
             # Transition (and exit) flow are stored in a 2D FxC ndarray where f is the flow idx
-            # and c is the compartment idx, giving us a matrix of flows rates out of each compartment.
+            # and c is the compartment idx, giving us a matrix of flows rates out of each
+            # compartment.
             transition_flow_rates = np.zeros((len(self._flows), len(comp_vals)))
 
             # Entry flows are stored in 1D C sized ndarray with one element per compartment.
@@ -877,7 +1034,8 @@ class CompartmentalModel:
                     entry_flow_rates[flow.dest.idx] += flow_rates[flow_idx]
 
             # Convert flow rates to probabilities, and take a sample using these probabilities,
-            # so we end up with the changes to the compartment sizes due to flows over this timestep.
+            # so we end up with the changes to the compartment sizes due to flows over this
+            # timestep.
             entry_changes = stochastic.sample_entry_flows(seed, entry_flow_rates, self.timestep)
             transition_changes = stochastic.sample_transistion_flows(
                 seed, transition_flow_rates, flow_map, comp_vals, self.timestep
@@ -902,9 +1060,10 @@ class CompartmentalModel:
     def set_derived_outputs_whitelist(self, whitelist: List[str]):
         """
         Request that we should only calculate a subset of the model's derived outputs.
-        This can be useful when you only care about some results and you want to cut down on runtime.
-        For example, we may only need some derived outputs for calibration, but may need more later when we want to know
-        all the dyanmics that the model actually showed.
+        This can be useful when you only care about some results and you want to cut down on
+        runtime.
+        For example, we may only need some derived outputs for calibration, but may need more later
+        when we want to know all the dyanmics that the model actually showed.
 
         Args:
             whitelist: A list of the derived output names to calculate, ignoring all others.
@@ -935,10 +1094,11 @@ class CompartmentalModel:
             flows=self._flows,
             compartments=self.compartments,
             get_flow_rates=self._backend.get_flow_rates,
-            model = self,
+            model=self,
             whitelist=self._derived_outputs_whitelist,
             baseline=self._baseline,
-            idx_cache = self._derived_outputs_idx_cache
+            idx_cache=self._derived_outputs_idx_cache,
+            parameters=self._backend.parameters,
         )
 
     def request_output_for_flow(
@@ -960,8 +1120,9 @@ class CompartmentalModel:
             source_strata (optional): A whitelist of strata to filter the source compartments.
             dest_strata (optional): A whitelist of strata to filter the destination compartments.
             save_results (optional): Whether to save or discard the results. Defaults to ``True``.
-            raw_results (optional): Whether to use raw interpolated flow rates, or post-process them so that they're more
-                represenative of the changes in compartment sizes. Defaults to ``False``.
+            raw_results (optional): Whether to use raw interpolated flow rates, or post-process them
+                so that they're more representative of the changes in compartment sizes.
+                Defaults to ``False``.
         """
         source_strata = source_strata or {}
         dest_strata = dest_strata or {}
@@ -973,7 +1134,7 @@ class CompartmentalModel:
                 [f.is_match(flow_name, source_strata, dest_strata) for f in self._flows]
             )
             assert is_flow_exists, f"No flow matches: {flow_name} {source_strata} {dest_strata}"
-        
+
         self._derived_output_graph.add_node(name)
         self._derived_output_requests[name] = {
             "request_type": DerivedOutputRequest.FLOW,
@@ -1007,10 +1168,13 @@ class CompartmentalModel:
             msg = f"A derived output named {name} already exists."
             assert name not in self._derived_output_requests, msg
             is_match_exists = any(
-                [any([c.is_match(name, strata) for name in compartments]) for c in self.compartments]
+                [
+                    any([c.is_match(name, strata) for name in compartments])
+                    for c in self.compartments
+                ]
             )
             assert is_match_exists, f"No compartment matches: {compartments} {strata}"
-    
+
         self._derived_output_graph.add_node(name)
         self._derived_output_requests[name] = {
             "request_type": DerivedOutputRequest.COMPARTMENT,
@@ -1026,7 +1190,8 @@ class CompartmentalModel:
         save_results: bool = True,
     ):
         """
-        Adds a derived output to the model's results. The output will be the aggregate of other derived outputs.
+        Adds a derived output to the model's results. The output will be the aggregate of other
+        derived outputs.
 
         Args:
             name: The name of the derived output.
@@ -1057,13 +1222,13 @@ class CompartmentalModel:
         save_results: bool = True,
     ):
         """
-        Adds a derived output to the model's results. The output will be the accumulated values of another derived
-        output over the model's time period.
+        Adds a derived output to the model's results. The output will be the accumulated values of
+        another derived output over the model's time period.
 
         Args:
             name: The name of the derived output.
             source: The name of the derived outputs to accumulate.
-            start_time (optional): The time to start accumulating from, defaults to model start time.
+            start_time (optional): The time to start accumulating from, defaults to model start time
             save_results (optional): Whether to save or discard the results.
 
         """
@@ -1093,7 +1258,7 @@ class CompartmentalModel:
         Args:
             name: The name of the derived output.
             func: A function used to calculate the derived ouput.
-            sources: The derived ouputs to input into the function.
+            sources: The derived outputs to input into the function.
             save_results (optional): Whether to save or discard the results.
 
         Example:
@@ -1136,11 +1301,35 @@ class CompartmentalModel:
             "save_results": save_results,
         }
 
-    def request_computed_value_output(
-        self,
-        name: str,
-        save_results: bool = True
+    def request_param_function_output(
+        self, name: str, func: params.Function, save_results: bool = True
     ):
+        """Request a generic Function output
+
+        Args:
+            name (str): _description_
+            func (Function): The Function, whose args are Param
+            save_results (bool, optional): _description_. Defaults to True.
+        """
+
+        msg = f"A derived output named {name} already exists."
+        assert name not in self._derived_output_requests, msg
+        for k, v in func.kwargs.items():
+            if isinstance(v, params.DerivedOutput):
+                source = v.name
+                assert (
+                    source in self._derived_output_requests
+                ), f"Source {source} has not been requested."
+                self._derived_output_graph.add_edge(source, name)
+
+        self._derived_output_graph.add_node(name)
+        self._derived_output_requests[name] = {
+            "request_type": DerivedOutputRequest.PARAM_FUNCTION,
+            "func": func,
+            "save_results": save_results,
+        }
+
+    def request_computed_value_output(self, name: str, save_results: bool = True):
         """
         Save a computed value process output to derived outputs
 
@@ -1160,25 +1349,31 @@ class CompartmentalModel:
 
     def add_computed_value_process(self, name: str, processor: ComputedValueProcessor):
         """
-        Calculate (at runtime) values derived from the current compartment values and/or functions/input data
-        providing time varying shared values.  The output values of these processes can be used by function 
-        parameters, adjustments, and function flows.
+        Calculate (at runtime) values derived from the current compartment values and/or
+        functions/input data
+        providing time varying shared values.  The output values of these processes can be used by
+        function parameters, adjustments, and function flows.
 
         Args:
             name (str): Name (key) of derived value (use this when referencing it in functions)
             processor (DerivedValueProcessor): Object providing implementation
         """
+        # FIXME: We might actually have to keep this for now, at least until modellers get sick of
+        # seeing it and change over all the code
+        warn(
+            "Deprecated feature - use model.add_computed_value_func instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         self._computed_value_processors[name] = processor
 
-    def add_adjustment_system(self, name: str, system: AdjustmentSystem):
-        """Add an AdjustmentSystem that will process common adjustments in parallel
-        These provide equivalent functionality to individual flow adjustments (adjust.py)
-       
-        Args:
-            name (str): The name of the system used for lookup purpsoes
-            system (AdjustmentSystem): AdjustmentSystem object containing the implementation details
-        """
-        self._adjustment_systems[name] = system
+    def add_computed_value_func(self, name: str, func: params.Function):
+        if name in self._computed_values_graph_dict:
+            raise Exception(f"Computed value function with name {name} already exists")
+        self._computed_values_graph_dict[name] = func
+
+    def get_computed_value_keys(self):
+        return list(self._computed_value_processors) + list(self._computed_values_graph_dict)
 
     def _get_ref_idx(self):
         if self.ref_date:
